@@ -68,7 +68,7 @@
 /* COMPATIBILITY CONTROLS
 /* .ad
 /* .fi
-/* .IP "\fBundisclosed_recipients_header (To: undisclosed-recipients:;)\fR"
+/* .IP "\fBundisclosed_recipients_header (see 'postconf -d' output)\fR"
 /*	Message header that the Postfix \fBcleanup\fR(8) server inserts when a
 /*	message contains no To: or Cc: message header.
 /* .PP
@@ -84,6 +84,10 @@
 /* .IP "\fBalways_add_missing_headers (no)\fR"
 /*	Always add (Resent-) From:, To:, Date: or Message-ID: headers
 /*	when not present.
+/* .PP
+/*	Available in Postfix version 2.9 and later:
+/* .IP "\fBenable_long_queue_ids (no)\fR"
+/*	Enable long, non-repeating, queue IDs (queue file names).
 /* BUILT-IN CONTENT FILTERING CONTROLS
 /* .ad
 /* .fi
@@ -486,8 +490,15 @@ static void cleanup_service(VSTREAM *src, char *unused_service, char **argv)
      */
     if (CLEANUP_OUT_OK(state) == 0 && type > 0) {
 	while (type != REC_TYPE_END
-	       && (type = rec_get(src, buf, 0)) > 0)
-	     /* void */ ;
+	       && (type = rec_get(src, buf, 0)) > 0) {
+	    if (type == REC_TYPE_MILT_COUNT) {
+		int     milter_count = atoi(vstring_str(buf));
+
+		/* Avoid deadlock. */
+		if (milter_count >= 0)
+		    cleanup_milter_receive(state, milter_count);
+	    }
+	}
     }
 
     /*
