@@ -199,7 +199,6 @@ void    cleanup_control(CLEANUP_STATE *state, int flags)
 
 int     cleanup_flush(CLEANUP_STATE *state)
 {
-    const char *myname = "cleanup_flush";
     int     status;
     char   *junk;
     VSTRING *trace_junk;
@@ -239,17 +238,9 @@ int     cleanup_flush(CLEANUP_STATE *state)
     /*
      * Update the preliminary message size and count fields with the actual
      * values.
-     * 
-     * XXX This ugly code was moved up here from a lower-level module.
      */
-    if (vstream_fseek(state->dst, 0L, SEEK_SET) < 0)
-	msg_fatal("%s: vstream_fseek %s: %m", myname, cleanup_path);
-    cleanup_out_format(state, REC_TYPE_SIZE, REC_TYPE_SIZE_FORMAT,
-	    (REC_TYPE_SIZE_CAST1) (state->xtra_offset - state->data_offset),
-		       (REC_TYPE_SIZE_CAST2) state->data_offset,
-		       (REC_TYPE_SIZE_CAST3) state->rcpt_count,
-		       (REC_TYPE_SIZE_CAST4) state->qmgr_opts,
-		       (REC_TYPE_SIZE_CAST5) state->cont_length);
+    if (CLEANUP_OUT_OK(state))
+	cleanup_final(state);
 
     /*
      * If there was an error that requires us to generate a bounce message
@@ -306,8 +297,8 @@ int     cleanup_flush(CLEANUP_STATE *state)
 #endif
 	    mail_stream_ctl(state->handle,
 			    MAIL_STREAM_CTL_QUEUE, state->queue_name,
-			    MAIL_STREAM_CTL_CLASS, 0,
-			    MAIL_STREAM_CTL_SERVICE, 0,
+			    MAIL_STREAM_CTL_CLASS, (char *) 0,
+			    MAIL_STREAM_CTL_SERVICE, (char *) 0,
 #ifdef DELAY_ACTION
 			    MAIL_STREAM_CTL_DELAY, state->defer_delay,
 #endif
