@@ -78,7 +78,8 @@
 
 /* cleanup_masquerade_external - masquerade address external form */
 
-int     cleanup_masquerade_external(VSTRING *addr, ARGV *masq_domains)
+int     cleanup_masquerade_external(CLEANUP_STATE *state, VSTRING *addr,
+				            ARGV *masq_domains)
 {
     char   *domain;
     ssize_t domain_len;
@@ -108,6 +109,11 @@ int     cleanup_masquerade_external(VSTRING *addr, ARGV *masq_domains)
 	name = mystrndup(STR(addr), domain - 1 - STR(addr));
 	excluded = (string_list_match(cleanup_masq_exceptions, lowercase(name)) != 0);
 	myfree(name);
+	if (cleanup_masq_exceptions->error) {
+	    msg_info("%s: %s lookup error -- deferring delivery",
+		     state->queue_id, VAR_MASQ_EXCEPTIONS);
+	    state->errs |= CLEANUP_STAT_WRITE;
+	}
 	if (excluded)
 	    return (0);
     }
@@ -145,13 +151,14 @@ int     cleanup_masquerade_external(VSTRING *addr, ARGV *masq_domains)
 
 /* cleanup_masquerade_tree - masquerade address node */
 
-int     cleanup_masquerade_tree(TOK822 *tree, ARGV *masq_domains)
+int     cleanup_masquerade_tree(CLEANUP_STATE *state, TOK822 *tree,
+				        ARGV *masq_domains)
 {
     VSTRING *temp = vstring_alloc(100);
     int     did_rewrite;
 
     tok822_externalize(temp, tree->head, TOK822_STR_DEFL);
-    did_rewrite = cleanup_masquerade_external(temp, masq_domains);
+    did_rewrite = cleanup_masquerade_external(state, temp, masq_domains);
     tok822_free_tree(tree->head);
     tree->head = tok822_scan(STR(temp), &tree->tail);
 
@@ -161,13 +168,14 @@ int     cleanup_masquerade_tree(TOK822 *tree, ARGV *masq_domains)
 
 /* cleanup_masquerade_internal - masquerade address internal form */
 
-int     cleanup_masquerade_internal(VSTRING *addr, ARGV *masq_domains)
+int     cleanup_masquerade_internal(CLEANUP_STATE *state, VSTRING *addr,
+				            ARGV *masq_domains)
 {
     VSTRING *temp = vstring_alloc(100);
     int     did_rewrite;
 
     quote_822_local(temp, STR(addr));
-    did_rewrite = cleanup_masquerade_external(temp, masq_domains);
+    did_rewrite = cleanup_masquerade_external(state, temp, masq_domains);
     unquote_822_local(addr, STR(temp));
 
     vstring_free(temp);
@@ -190,13 +198,14 @@ int     main(int argc, char **argv)
 {
     VSTRING *addr;
     ARGV   *masq_domains;
+    CLEANUP_STATE state;
 
     if (argc != 4)
 	msg_fatal("usage: %s exceptions masquerade_list address", argv[0]);
 
     var_masq_exceptions = argv[1];
     cleanup_masq_exceptions =
-	string_list_init(MATCH_FLAG_NONE, var_masq_exceptions);
+	string_list_init(MATCH_FLAG_RETURN, var_masq_exceptions);
     masq_domains = argv_split(argv[2], " ,\t\r\n");
     addr = vstring_alloc(1);
     if (strchr(argv[3], '@') == 0)
@@ -208,9 +217,11 @@ int     main(int argc, char **argv)
     vstream_printf("masq_list:  %s\n", argv[2]);
     vstream_printf("address:    %s\n", argv[3]);
 
-    cleanup_masquerade_external(addr, masq_domains);
+    state.errs = 0;
+    cleanup_masquerade_external(&state, addr, masq_domains);
 
     vstream_printf("result:     %s\n", STR(addr));
+    vstream_printf("errs:       %d\n", state.errs);
     vstream_fflush(VSTREAM_OUT);
 
     vstring_free(addr);
