@@ -169,7 +169,7 @@ static void cleanup_rewrite_sender(CLEANUP_STATE *state,
 	    if (cleanup_masq_domains
 		&& (cleanup_masq_flags & CLEANUP_MASQ_FLAG_HDR_FROM))
 		did_rewrite |=
-		    cleanup_masquerade_tree(*tpp, cleanup_masq_domains);
+		    cleanup_masquerade_tree(state, *tpp, cleanup_masq_domains);
 	}
     }
     if (did_rewrite) {
@@ -226,7 +226,7 @@ static void cleanup_rewrite_recip(CLEANUP_STATE *state,
 	    if (cleanup_masq_domains
 		&& (cleanup_masq_flags & CLEANUP_MASQ_FLAG_HDR_RCPT))
 		did_rewrite |=
-		    cleanup_masquerade_tree(*tpp, cleanup_masq_domains);
+		    cleanup_masquerade_tree(state, *tpp, cleanup_masq_domains);
 	}
     }
     if (did_rewrite) {
@@ -479,10 +479,6 @@ static void cleanup_header_callback(void *context, int header_class,
     if (hdr_opts && (hdr_opts->flags & HDR_OPT_MIME))
 	header_class = MIME_HDR_MULTIPART;
 
-    /* Update the Received: header count before maybe dropping headers below. */
-    if (hdr_opts && hdr_opts->type == HDR_RECEIVED)
-	state->hop_count += 1;
-
     if ((state->flags & CLEANUP_FLAG_FILTER)
 	&& (CHECK(MIME_HDR_PRIMARY, cleanup_header_checks, VAR_HEADER_CHECKS)
     || CHECK(MIME_HDR_MULTIPART, cleanup_mimehdr_checks, VAR_MIMEHDR_CHECKS)
@@ -502,6 +498,10 @@ static void cleanup_header_callback(void *context, int header_class,
 		hdr_opts = header_opts_find(result);
 		myfree((char *) result);
 	    }
+	} else if (checks->error) {
+	    msg_warn("%s: %s map lookup problem -- deferring delivery",
+		     state->queue_id, checks->title);
+	    state->errs |= CLEANUP_STAT_WRITE;
 	}
     }
 
@@ -578,13 +578,9 @@ static void cleanup_header_callback(void *context, int header_class,
 	    msg_info("%s: message-id=%s", state->queue_id, hdrval);
 	if (hdr_opts->type == HDR_RESENT_MESSAGE_ID)
 	    msg_info("%s: resent-message-id=%s", state->queue_id, hdrval);
-	if (hdr_opts->type == HDR_RECEIVED) {
-	    if (state->hop_count >= var_hopcount_limit)
+	if (hdr_opts->type == HDR_RECEIVED)
+	    if (++state->hop_count >= var_hopcount_limit)
 		state->errs |= CLEANUP_STAT_HOPS;
-	    /* Save our Received: header after maybe updating headers above. */
-	    if (state->hop_count == 1)
-		argv_add(state->auto_hdrs, vstring_str(header_buf), ARGV_END);
-	}
 	if (CLEANUP_OUT_OK(state)) {
 	    if (hdr_opts->flags & HDR_OPT_RR)
 		state->resent = "Resent-";
@@ -644,14 +640,21 @@ static void cleanup_header_done_callback(void *context)
     if ((state->hdr_rewrite_context || var_always_add_hdrs)
 	&& (state->headers_seen & (1 << (state->resent[0] ?
 			   HDR_RESENT_MESSAGE_ID : HDR_MESSAGE_ID))) == 0) {
-	tv = state->handle->ctime.tv_sec;
-	tp = gmtime(&tv);
-	strftime(time_stamp, sizeof(time_stamp), "%Y%m%d%H%M%S", tp);
-	cleanup_out_format(state, REC_TYPE_NORM, "%sMessage-Id: <%s.%s@%s>",
-		state->resent, time_stamp, state->queue_id, var_myhostname);
-	msg_info("%s: %smessage-id=<%s.%s@%s>",
+	if (var_long_queue_ids) {
+	    vstring_sprintf(state->temp1, "%s@%s",
+			    state->queue_id, var_myhostname);
+	} else {
+	    tv = state->handle->ctime.tv_sec;
+	    tp = gmtime(&tv);
+	    strftime(time_stamp, sizeof(time_stamp), "%Y%m%d%H%M%S", tp);
+	    vstring_sprintf(state->temp1, "%s.%s@%s",
+			    time_stamp, state->queue_id, var_myhostname);
+	}
+	cleanup_out_format(state, REC_TYPE_NORM, "%sMessage-Id: <%s>",
+			   state->resent, vstring_str(state->temp1));
+	msg_info("%s: %smessage-id=<%s>",
 		 state->queue_id, *state->resent ? "resent-" : "",
-		 time_stamp, state->queue_id, var_myhostname);
+		 vstring_str(state->temp1));
 	state->headers_seen |= (1 << (state->resent[0] ?
 				   HDR_RESENT_MESSAGE_ID : HDR_MESSAGE_ID));
     }
@@ -785,6 +788,10 @@ static void cleanup_body_callback(void *context, int type,
 		myfree((char *) result);
 		return;
 	    }
+	} else if (cleanup_body_checks->error) {
+	    msg_warn("%s: %s map lookup problem -- deferring delivery",
+		     state->queue_id, cleanup_body_checks->title);
+	    state->errs |= CLEANUP_STAT_WRITE;
 	}
     }
     cleanup_out(state, type, buf, len);
@@ -822,6 +829,8 @@ static void cleanup_message_headerbody(CLEANUP_STATE *state, int type,
     /*
      * Strip unwanted characters. Don't overwrite the input.
      * 
+     * XXX Possible space+time optimization: use a bitset.
+     * 
      * XXX Possible optimization: simplify the loop when the "strip" set
      * contains only one character.
      * 
