@@ -186,7 +186,7 @@
 #include <safe.h>
 #include <connect.h>
 #include <valid_hostname.h>
-#include <events.h>
+#include <warn_stat.h>
 
 /* Global library. */
 
@@ -352,7 +352,6 @@ static void flush_queue(void)
     if (mail_flush_maildrop() < 0)
 	msg_fatal_status(EX_UNAVAILABLE,
 			 "Cannot flush mail queue - mail system is down");
-    event_drain(2);
 }
 
 /* flush_site - flush mail for site */
@@ -479,6 +478,11 @@ int     main(int argc, char **argv)
     msg_syslog_init(mail_task("postqueue"), LOG_PID, LOG_FACILITY);
     set_mail_conf_str(VAR_PROCNAME, var_procname = mystrdup(argv[0]));
 
+    /*
+     * Check the Postfix library version as soon as we enable logging.
+     */
+    MAIL_VERSION_CHECK;
+
     /*
      * Parse JCL. This program is set-gid and must sanitize all command-line
      * parameters. The configuration directory argument is validated by the
