@@ -123,6 +123,7 @@
 #include <debug_peer.h>
 #include <mail_params.h>
 #include <maps.h>
+#include <smtp_stream.h>
 
 /* Application-specific. */
 
@@ -196,6 +197,8 @@ static void tls_site_lookup(int *site_level, const char *site_name,
 	    msg_warn("Table %s: ignoring unknown TLS policy '%s' for %s %s",
 		     var_smtp_tls_per_site, lookup, site_class, site_name);
 	}
+    } else if (tls_per_site->error) {
+	msg_fatal("%s lookup error for %s", tls_per_site->title, site_name);
     }
 }
 
@@ -220,6 +223,12 @@ static int tls_policy_lookup_one(SMTP_SESSION *session, int *site_level,
     if ((lookup = maps_find(tls_policy, site_name, 0)) == 0)
 	return (0);
 
+    if (tls_policy->error) {
+	msg_warn("%s: %s lookup error for %s",
+		 session->state->request->queue_id,
+		 tls_policy->title, site_name);
+	vstream_longjmp(session->stream, SMTP_ERR_DATA);
+    }
     if (cbuf == 0)
 	cbuf = vstring_alloc(10);
 
