@@ -335,6 +335,15 @@ typedef struct SINK_STATE {
 #define SOFT_ERROR_RESP		"450 4.3.0 Error: command failed"
 #define HARD_ERROR_RESP		"500 5.3.0 Error: command failed"
 
+ /*
+  * We can't rely on vstream auto-flushing, so we have to prepare for the
+  * next read request.
+  */
+#define SMTP_FLUSH(fp) do { \
+    if (vstream_peek(fp) <= 0 && readable(vstream_fileno(fp)) <= 0) \
+        smtp_flush(fp); \
+    } while (0)
+
 static int var_tmout = 100;
 static int var_max_line_length = 2048;
 static char *var_myhostname;
@@ -386,16 +395,16 @@ static void do_stats(void)
 
 static void hard_err_resp(SINK_STATE *state)
 {
-    smtp_printf(state->stream, hard_error_resp);
-    smtp_flush(state->stream);
+    smtp_printf(state->stream, "%s", hard_error_resp);
+    SMTP_FLUSH(state->stream);
 }
 
 /* soft_err_resp - generic soft error response */
 
 static void soft_err_resp(SINK_STATE *state)
 {
-    smtp_printf(state->stream, soft_error_resp);
-    smtp_flush(state->stream);
+    smtp_printf(state->stream, "%s", soft_error_resp);
+    SMTP_FLUSH(state->stream);
 }
 
 /* exp_path_template - expand template pathname, static result */
@@ -616,8 +625,9 @@ static void ehlo_response(SINK_STATE *state, const char *args)
 	smtp_printf(state->stream, "250-XFORWARD NAME ADDR PROTO HELO");
     if (!disable_enh_status)
 	smtp_printf(state->stream, "250-ENHANCEDSTATUSCODES");
+    /* RFC 821/2821/5321: Format is replycode<SPACE>optional-text<CRLF> */
     smtp_printf(state->stream, "250 ");
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
     if (single_template) {
 	if (state->helo_args)
 	    myfree(state->helo_args);
@@ -634,7 +644,7 @@ static void helo_response(SINK_STATE *state, const char *args)
     mail_cmd_reset(state);
     state->client_proto = "SMTP";
     smtp_printf(state->stream, "250 %s", var_myhostname);
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
     if (single_template) {
 	if (state->helo_args)
 	    myfree(state->helo_args);
@@ -648,7 +658,7 @@ static void helo_response(SINK_STATE *state, const char *args)
 static void ok_response(SINK_STATE *state, const char *unused_args)
 {
     smtp_printf(state->stream, "250 2.0.0 Ok");
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
 }
 
 /* rset_response - reset, send 250 OK */
@@ -657,7 +667,7 @@ static void rset_response(SINK_STATE *state, const char *unused_args)
 {
     mail_cmd_reset(state);
     smtp_printf(state->stream, "250 2.1.0 Ok");
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
 }
 
 /* mail_response - reset recipient count, send 250 OK */
@@ -666,13 +676,13 @@ static void mail_response(SINK_STATE *state, const char *args)
 {
     if (state->in_mail) {
 	smtp_printf(state->stream, "503 5.5.1 Error: nested MAIL command");
-	smtp_flush(state->stream);
+	SMTP_FLUSH(state->stream);
 	return;
     }
     state->in_mail++;
     state->rcpts = 0;
     smtp_printf(state->stream, "250 2.1.0 Ok");
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
     if (single_template) {
 	mail_file_open(state);
 	SKIP(args, *args != ':');
@@ -688,12 +698,12 @@ static void rcpt_response(SINK_STATE *state, const char *args)
 {
     if (state->in_mail == 0) {
 	smtp_printf(state->stream, "503 5.5.1 Error: need MAIL command");
-	smtp_flush(state->stream);
+	SMTP_FLUSH(state->stream);
 	return;
     }
     state->rcpts++;
     smtp_printf(state->stream, "250 2.1.5 Ok");
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
     /* Note: there may be more than one recipient per mail transaction. */
     if (state->dump_file) {
 	SKIP(args, *args != ':');
@@ -710,7 +720,7 @@ static void abort_event(int unused_event, char *context)
     SINK_STATE *state = (SINK_STATE *) context;
 
     smtp_printf(state->stream, "550 This violates SMTP");
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
     disconnect(state);
 }
 
@@ -720,13 +730,13 @@ static void data_response(SINK_STATE *state, const char *unused_args)
 {
     if (state->in_mail == 0 || state->rcpts == 0) {
 	smtp_printf(state->stream, "503 5.5.1 Error: need RCPT command");
-	smtp_flush(state->stream);
+	SMTP_FLUSH(state->stream);
 	return;
     }
     /* Not: ST_ANY. */
     state->data_state = ST_CR_LF;
     smtp_printf(state->stream, "354 End data with <CR><LF>.<CR><LF>");
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
     if (abort_delay < 0) {
 	state->read_fn = data_read;
     } else {
@@ -745,11 +755,11 @@ static void dot_resp_hard(SINK_STATE *state)
 {
     if (enable_lmtp) {
 	while (state->rcpts-- > 0)	/* XXX this could block */
-	    smtp_printf(state->stream, hard_error_resp);
+	    smtp_printf(state->stream, "%s", hard_error_resp);
     } else {
-	smtp_printf(state->stream, hard_error_resp);
+	smtp_printf(state->stream, "%s", hard_error_resp);
     }
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
 }
 
 /* dot_resp_soft - soft error response to . command */
@@ -758,11 +768,11 @@ static void dot_resp_soft(SINK_STATE *state)
 {
     if (enable_lmtp) {
 	while (state->rcpts-- > 0)	/* XXX this could block */
-	    smtp_printf(state->stream, soft_error_resp);
+	    smtp_printf(state->stream, "%s", soft_error_resp);
     } else {
-	smtp_printf(state->stream, soft_error_resp);
+	smtp_printf(state->stream, "%s", soft_error_resp);
     }
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
 }
 
 /* dot_response - response to . command */
@@ -775,7 +785,7 @@ static void dot_response(SINK_STATE *state, const char *unused_args)
     } else {
 	smtp_printf(state->stream, "250 2.0.0 Ok");
     }
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
 }
 
 /* quit_response - respond to QUIT command */
@@ -783,7 +793,7 @@ static void dot_response(SINK_STATE *state, const char *unused_args)
 static void quit_response(SINK_STATE *state, const char *unused_args)
 {
     smtp_printf(state->stream, "221 Bye");
-    smtp_flush(state->stream);
+    smtp_flush(state->stream);			/* not: SMTP_FLUSH */
     if (count)
 	quit_count++;
 }
@@ -798,7 +808,7 @@ static void conn_response(SINK_STATE *state, const char *unused_args)
 	smtp_printf(state->stream, "220 %s", var_myhostname);
     else
 	smtp_printf(state->stream, "220 %s ESMTP", var_myhostname);
-    smtp_flush(state->stream);
+    SMTP_FLUSH(state->stream);
 }
 
 /* delay_event - delayed command response */
@@ -1174,7 +1184,7 @@ static int command_read(SINK_STATE *state)
 	msg_info("%s", ptr);
     if ((command = mystrtok(&ptr, " \t")) == 0) {
 	smtp_printf(state->stream, "500 5.5.2 Error: unknown command");
-	smtp_flush(state->stream);
+	SMTP_FLUSH(state->stream);
 	return (0);
     }
     for (cmdp = command_table; cmdp->name != 0; cmdp++)
@@ -1182,7 +1192,7 @@ static int command_read(SINK_STATE *state)
 	    break;
     if (cmdp->name == 0 || (cmdp->flags & FLAG_ENABLE) == 0) {
 	smtp_printf(state->stream, "500 5.5.1 Error: unknown command");
-	smtp_flush(state->stream);
+	SMTP_FLUSH(state->stream);
 	return (0);
     }
     return (command_resp(state, cmdp, command, printable(ptr, '?')));
