@@ -18,14 +18,15 @@
 /*	bool    var_tls_append_def_CA;
 /*	bool	var_tls_preempt_clist;
 /*
-/*	TLS_APPL_STATE *tls_alloc_app_context(ssl_ctx)
+/*	TLS_APPL_STATE *tls_alloc_app_context(ssl_ctx, log_mask)
 /*	SSL_CTX	*ssl_ctx;
+/*	int	log_mask;
 /*
 /*	void	tls_free_app_context(app_ctx)
 /*	void	*app_ctx;
 /*
-/*	TLS_SESS_STATE *tls_alloc_sess_context(log_level, namaddr)
-/*	int	log_level;
+/*	TLS_SESS_STATE *tls_alloc_sess_context(log_mask, namaddr)
+/*	int	log_mask;
 /*	const char *namaddr;
 /*
 /*	void	tls_free_context(TLScontext)
@@ -66,6 +67,10 @@
 /*	int	argi;
 /*	long	argl; /* unused */
 /*	long	ret;
+/*
+/*	int	tls_log_mask(log_param, log_level)
+/*	const char *log_param;
+/*	const char *log_level;
 /* DESCRIPTION
 /*	This module implements routines that support the TLS client
 /*	and server internals.
@@ -126,6 +131,10 @@
 /*	tls_bio_dump_cb() is a call-back routine for the
 /*	BIO_set_callback() routine. It logs SSL content to the
 /*	Postfix logfile.
+/*
+/*	tls_log_mask() converts a TLS log_level value from string
+/*	to mask.  The main.cf parameter name is passed along for
+/*	diagnostics.
 /* LICENSE
 /* .ad
 /* .fi
@@ -215,12 +224,6 @@ static const NAME_CODE protocol_table[] = {
     SSL_TXT_SSLV2, TLS_PROTOCOL_SSLv2,
     SSL_TXT_SSLV3, TLS_PROTOCOL_SSLv3,
     SSL_TXT_TLSV1, TLS_PROTOCOL_TLSv1,
-#ifdef SSL_TXT_TLSV1_1
-    SSL_TXT_TLSV1_1, TLS_PROTOCOL_TLSv1_1,
-#endif
-#ifdef SSL_TXT_TLSV1_2
-    SSL_TXT_TLSV1_2, TLS_PROTOCOL_TLSv1_2,
-#endif
     0, TLS_PROTOCOL_INVALID,
 };
 
@@ -230,72 +233,59 @@ static const NAME_CODE protocol_table[] = {
 #define NAMEBUG(x)	#x, SSL_OP_##x
 static const LONG_NAME_MASK ssl_bug_tweaks[] = {
 
-#ifndef SSL_OP_MICROSOFT_SESS_ID_BUG
-#define SSL_OP_MICROSOFT_SESS_ID_BUG		0
+#if defined(SSL_OP_MICROSOFT_SESS_ID_BUG)
+    NAMEBUG(MICROSOFT_SESS_ID_BUG),	/* 0x00000001L */
 #endif
-    NAMEBUG(MICROSOFT_SESS_ID_BUG),
 
-#ifndef SSL_OP_NETSCAPE_CHALLENGE_BUG
-#define SSL_OP_NETSCAPE_CHALLENGE_BUG		0
+#if defined(SSL_OP_NETSCAPE_CHALLENGE_BUG)
+    NAMEBUG(NETSCAPE_CHALLENGE_BUG),	/* 0x00000002L */
 #endif
-    NAMEBUG(NETSCAPE_CHALLENGE_BUG),
 
-#ifndef SSL_OP_LEGACY_SERVER_CONNECT
-#define SSL_OP_LEGACY_SERVER_CONNECT		0
+#if defined(SSL_OP_LEGACY_SERVER_CONNECT)
+    NAMEBUG(LEGACY_SERVER_CONNECT),	/* 0x00000004L */
 #endif
-    NAMEBUG(LEGACY_SERVER_CONNECT),
 
-#ifndef SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG
-#define SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG 0
-#endif
-    NAMEBUG(NETSCAPE_REUSE_CIPHER_CHANGE_BUG),
+#if defined(SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG)
+    NAMEBUG(NETSCAPE_REUSE_CIPHER_CHANGE_BUG),	/* 0x00000008L */
     "CVE-2010-4180", SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG,
-
-#ifndef SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG
-#define SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG	0
 #endif
-    NAMEBUG(SSLREF2_REUSE_CERT_TYPE_BUG),
 
-#ifndef SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER
-#define SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER	0
+#if defined(SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG)
+    NAMEBUG(SSLREF2_REUSE_CERT_TYPE_BUG),	/* 0x00000010L */
 #endif
-    NAMEBUG(MICROSOFT_BIG_SSLV3_BUFFER),
 
-#ifndef SSL_OP_MSIE_SSLV2_RSA_PADDING
-#define SSL_OP_MSIE_SSLV2_RSA_PADDING		0
+#if defined(SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER)
+    NAMEBUG(MICROSOFT_BIG_SSLV3_BUFFER),/* 0x00000020L	 */
 #endif
-    NAMEBUG(MSIE_SSLV2_RSA_PADDING),
+
+#if defined(SSL_OP_MSIE_SSLV2_RSA_PADDING)
+    NAMEBUG(MSIE_SSLV2_RSA_PADDING),	/* 0x00000040L */
     "CVE-2005-2969", SSL_OP_MSIE_SSLV2_RSA_PADDING,
-
-#ifndef SSL_OP_SSLEAY_080_CLIENT_DH_BUG
-#define SSL_OP_SSLEAY_080_CLIENT_DH_BUG		0
 #endif
-    NAMEBUG(SSLEAY_080_CLIENT_DH_BUG),
 
-#ifndef SSL_OP_TLS_D5_BUG
-#define SSL_OP_TLS_D5_BUG			0
+#if defined(SSL_OP_SSLEAY_080_CLIENT_DH_BUG)
+    NAMEBUG(SSLEAY_080_CLIENT_DH_BUG),	/* 0x00000080L */
 #endif
-    NAMEBUG(TLS_D5_BUG),
 
-#ifndef SSL_OP_TLS_BLOCK_PADDING_BUG
-#define SSL_OP_TLS_BLOCK_PADDING_BUG		0
+#if defined(SSL_OP_TLS_D5_BUG)
+    NAMEBUG(TLS_D5_BUG),		/* 0x00000100L	 */
 #endif
-    NAMEBUG(TLS_BLOCK_PADDING_BUG),
 
-#ifndef SSL_OP_TLS_ROLLBACK_BUG
-#define SSL_OP_TLS_ROLLBACK_BUG			0
+#if defined(SSL_OP_TLS_BLOCK_PADDING_BUG)
+    NAMEBUG(TLS_BLOCK_PADDING_BUG),	/* 0x00000200L */
 #endif
-    NAMEBUG(TLS_ROLLBACK_BUG),
 
-#ifndef SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS
-#define SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS	0
+#if defined(SSL_OP_TLS_ROLLBACK_BUG)
+    NAMEBUG(TLS_ROLLBACK_BUG),		/* 0x00000400L */
 #endif
-    NAMEBUG(DONT_INSERT_EMPTY_FRAGMENTS),
 
-#ifndef SSL_OP_CRYPTOPRO_TLSEXT_BUG
-#define SSL_OP_CRYPTOPRO_TLSEXT_BUG		0
+#if defined(SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS)
+    NAMEBUG(DONT_INSERT_EMPTY_FRAGMENTS),	/* 0x00000800L */
+#endif
+
+#if defined(SSL_OP_CRYPTOPRO_TLSEXT_BUG)
+    NAMEBUG(CRYPTOPRO_TLSEXT_BUG),	/* 0x80000000L */
 #endif
-    NAMEBUG(CRYPTOPRO_TLSEXT_BUG),
     0, 0,
 };
 
@@ -312,6 +302,43 @@ const NAME_CODE tls_cipher_grade_table[] = {
     0, TLS_CIPHER_NONE,
 };
 
+ /*
+  * Log keyword <=> mask conversion.
+  */
+#define TLS_LOG_0 TLS_LOG_NONE
+#define TLS_LOG_1 TLS_LOG_SUMMARY
+#define TLS_LOG_2 (TLS_LOG_1 | TLS_LOG_VERBOSE | TLS_LOG_CACHE | TLS_LOG_DEBUG)
+#define TLS_LOG_3 (TLS_LOG_2 | TLS_LOG_TLSPKTS)
+#define TLS_LOG_4 (TLS_LOG_3 | TLS_LOG_ALLPKTS)
+
+static const NAME_MASK tls_log_table[] = {
+    "0", TLS_LOG_0,
+    "none", TLS_LOG_NONE,
+    "1", TLS_LOG_1,
+    "routine", TLS_LOG_1,
+    "2", TLS_LOG_2,
+    "debug", TLS_LOG_2,
+    "3", TLS_LOG_3,
+    "ssl-expert", TLS_LOG_3,
+    "4", TLS_LOG_4,
+    "ssl-developer", TLS_LOG_4,
+    "5", TLS_LOG_4,			/* for good measure */
+    "6", TLS_LOG_4,			/* for good measure */
+    "7", TLS_LOG_4,			/* for good measure */
+    "8", TLS_LOG_4,			/* for good measure */
+    "9", TLS_LOG_4,			/* for good measure */
+    "summary", TLS_LOG_SUMMARY,
+    "untrusted", TLS_LOG_UNTRUSTED,
+    "peercert", TLS_LOG_PEERCERT,
+    "certmatch", TLS_LOG_CERTMATCH,
+    "verbose", TLS_LOG_VERBOSE,		/* Postfix TLS library verbose */
+    "cache", TLS_LOG_CACHE,
+    "ssl-debug", TLS_LOG_DEBUG,		/* SSL library debug/verbose */
+    "ssl-handshake-packet-dump", TLS_LOG_TLSPKTS,
+    "ssl-session-packet-dump", TLS_LOG_TLSPKTS | TLS_LOG_ALLPKTS,
+    0, 0,
+};
+
  /*
   * Parsed OpenSSL version number.
   */
@@ -339,6 +366,17 @@ static const cipher_probe_t cipher_probes[] = {
     0, 0, 0,
 };
 
+/* tls_log_mask - Convert user TLS loglevel to internal log feature mask */
+
+int     tls_log_mask(const char *log_param, const char *log_level)
+{
+    int     mask;
+
+    mask = name_mask_opt(log_param, tls_log_table, log_level,
+			 NAME_MASK_ANY_CASE | NAME_MASK_RETURN);
+    return (mask);
+}
+
 /* tls_exclude_missing - Append exclusions for missing ciphers */
 
 static const char *tls_exclude_missing(SSL_CTX *ctx, VSTRING *buf)
@@ -466,10 +504,8 @@ int     tls_protocol_mask(const char *plist)
 	else
 	    include |= code =
 		name_code(protocol_table, NAME_CODE_FLAG_NONE, tok);
-	if (code == TLS_PROTOCOL_INVALID) {
-	    myfree(save);
+	if (code == TLS_PROTOCOL_INVALID)
 	    return TLS_PROTOCOL_INVALID;
-	}
     }
     myfree(save);
 
@@ -623,14 +659,16 @@ const char *tls_set_ciphers(TLS_APPL_STATE *app_ctx, const char *context,
 
 /* tls_alloc_app_context - allocate TLS application context */
 
-TLS_APPL_STATE *tls_alloc_app_context(SSL_CTX *ssl_ctx)
+TLS_APPL_STATE *tls_alloc_app_context(SSL_CTX *ssl_ctx, int log_mask)
 {
     TLS_APPL_STATE *app_ctx;
 
     app_ctx = (TLS_APPL_STATE *) mymalloc(sizeof(*app_ctx));
 
+    /* See portability note below with other memset() call. */
     memset((char *) app_ctx, 0, sizeof(*app_ctx));
     app_ctx->ssl_ctx = ssl_ctx;
+    app_ctx->log_mask = log_mask;
 
     /* See also: cache purging code in tls_set_ciphers(). */
     app_ctx->cipher_grade = TLS_CIPHER_NONE;
@@ -662,7 +700,7 @@ void    tls_free_app_context(TLS_APPL_STATE *app_ctx)
 
 /* tls_alloc_sess_context - allocate TLS session context */
 
-TLS_SESS_STATE *tls_alloc_sess_context(int log_level, const char *namaddr)
+TLS_SESS_STATE *tls_alloc_sess_context(int log_mask, const char *namaddr)
 {
     TLS_SESS_STATE *TLScontext;
 
@@ -683,9 +721,10 @@ TLS_SESS_STATE *tls_alloc_sess_context(int log_level, const char *namaddr)
     TLScontext->peer_CN = 0;
     TLScontext->issuer_CN = 0;
     TLScontext->peer_fingerprint = 0;
+    TLScontext->peer_pkey_fprint = 0;
     TLScontext->protocol = 0;
     TLScontext->cipher_name = 0;
-    TLScontext->log_level = log_level;
+    TLScontext->log_mask = log_mask;
     TLScontext->namaddr = lowercase(mystrdup(namaddr));
     TLScontext->fpt_dgst = 0;
 
@@ -716,6 +755,8 @@ void    tls_free_context(TLS_SESS_STATE *TLScontext)
 	myfree(TLScontext->issuer_CN);
     if (TLScontext->peer_fingerprint)
 	myfree(TLScontext->peer_fingerprint);
+    if (TLScontext->peer_pkey_fprint)
+	myfree(TLScontext->peer_pkey_fprint);
     if (TLScontext->fpt_dgst)
 	myfree(TLScontext->fpt_dgst);
 
@@ -813,10 +854,8 @@ void    tls_check_version(void)
 long    tls_bug_bits(void)
 {
     long    bits = SSL_OP_ALL;		/* Work around all known bugs */
-    long    mask;
 
-#if OPENSSL_VERSION_NUMBER >= 0x00908000L && \
-	OPENSSL_VERSION_NUMBER < 0x10000000L
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L
     long    lib_version = SSLeay();
 
     /*
@@ -842,10 +881,6 @@ long    tls_bug_bits(void)
 	bits &= ~long_name_mask_opt(VAR_TLS_BUG_TWEAKS, ssl_bug_tweaks,
 				    var_tls_bug_tweaks, NAME_MASK_ANY_CASE |
 				    NAME_MASK_NUMBER | NAME_MASK_WARN);
-#ifdef SSL_OP_SAFARI_ECDHE_ECDSA_BUG
-	/* Not relevant to SMTP */
-	bits &= ~SSL_OP_SAFARI_ECDHE_ECDSA_BUG;
-#endif
     }
     return (bits);
 }
