@@ -60,6 +60,7 @@
 #include "stringops.h"
 #include "dict.h"
 #include "dict_cdb.h"
+#include "warn_stat.h"
 
 #ifdef HAS_CDB
 
@@ -103,7 +104,7 @@ static const char *dict_cdbq_lookup(DICT *dict, const char *name)
     static unsigned len;
     const char *result = 0;
 
-    dict_errno = 0;
+    dict->error = 0;
 
     /* CDB is constant, so do not try to acquire a lock. */
 
@@ -184,7 +185,8 @@ static DICT *dict_cdbq_open(const char *path, int dict_flags)
     cdb_path = concatenate(path, CDB_SUFFIX, (char *) 0);
 
     if ((fd = open(cdb_path, O_RDONLY)) < 0)
-	msg_fatal("open database %s: %m", cdb_path);
+	return (dict_surrogate(DICT_TYPE_CDB, path, O_RDONLY, dict_flags,
+			       "open database %s: %m", cdb_path));
 
     dict_cdbq = (DICT_CDBQ *) dict_alloc(DICT_TYPE_CDB,
 					 cdb_path, sizeof(*dict_cdbq));
@@ -200,6 +202,8 @@ static DICT *dict_cdbq_open(const char *path, int dict_flags)
     if (fstat(fd, &st) < 0)
 	msg_fatal("dict_dbq_open: fstat: %m");
     dict_cdbq->dict.mtime = st.st_mtime;
+    dict_cdbq->dict.owner.uid = st.st_uid;
+    dict_cdbq->dict.owner.status = (st.st_uid != 0);
     close_on_exec(fd, CLOSE_ON_EXEC);
 
     /*
@@ -227,12 +231,14 @@ static DICT *dict_cdbq_open(const char *path, int dict_flags)
 
 /* dict_cdbm_update - add database entry, create mode */
 
-static void dict_cdbm_update(DICT *dict, const char *name, const char *value)
+static int dict_cdbm_update(DICT *dict, const char *name, const char *value)
 {
     DICT_CDBM *dict_cdbm = (DICT_CDBM *) dict;
     unsigned ksize, vsize;
     int     r;
 
+    dict->error = 0;
+
     /*
      * Optionally fold the key.
      */
@@ -279,9 +285,11 @@ static void dict_cdbm_update(DICT *dict, const char *name, const char *value)
 	    msg_fatal("%s: duplicate entry: \"%s\"",
 		      dict_cdbm->dict.name, name);
     }
+    return (r);
 #else
     if (cdb_make_add(&dict_cdbm->cdbm, name, ksize, value, vsize) < 0)
 	msg_fatal("error writing %s: %m", dict_cdbm->tmp_path);
+    return (0);
 #endif
 }
 
@@ -333,9 +341,11 @@ static DICT *dict_cdbm_open(const char *path, int dict_flags)
      * isn't creating it at the same time.
      */
     for (;;) {
-	if ((fd = open(tmp_path, O_RDWR | O_CREAT, 0644)) < 0
-	    || fstat(fd, &st0) < 0)
-	    msg_fatal("open database %s: %m", tmp_path);
+	if ((fd = open(tmp_path, O_RDWR | O_CREAT, 0644)) < 0)
+	    return (dict_surrogate(DICT_TYPE_CDB, path, O_RDWR, dict_flags,
+				   "open database %s: %m", tmp_path));
+	if (fstat(fd, &st0) < 0)
+	    msg_fatal("fstat(%s): %m", tmp_path);
 
 	/*
 	 * Get an exclusive lock - we're going to change the database so we
@@ -373,6 +383,8 @@ static DICT *dict_cdbm_open(const char *path, int dict_flags)
     dict_cdbm->dict.update = dict_cdbm_update;
     dict_cdbm->cdb_path = cdb_path;
     dict_cdbm->tmp_path = tmp_path;
+    dict_cdbm->dict.owner.uid = st1.st_uid;
+    dict_cdbm->dict.owner.status = (st1.st_uid != 0);
     close_on_exec(fd, CLOSE_ON_EXEC);
 
     /*
